/******************************************************************************
 * Board.java - Define a backgammon board 
 * $Id: Board.java,v 1.31 2009/02/14 12:04:30 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Board.java,v $
 * Revision 1.31  2009/02/14 12:04:30  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.30  2009/02/12 06:05:26  dick
 * Javadoc.
 *
 * Revision 1.29  2009/02/11 08:59:24  dick
 * Better beginning and ending your turn.
 *
 * Revision 1.28  2009/02/06 07:56:49  dick
 * Working on doubles.
 *
 * Revision 1.27  2009/02/01 21:21:01  dick
 * Add savedMatches and missManners lists to track other players' behavior.
 *
 * Revision 1.26  2009/01/29 08:26:39  dick
 * Fix the pip count calculation.
 *
 * Revision 1.25  2009/01/28 19:42:37  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.24  2009/01/26 07:31:24  dick
 * Sort the dice high to low.
 *
 * Revision 1.23  2009/01/24 17:04:21  dick
 * When parsing the board, i tried to normalize the data so that the player was always playing 24-1.
 * That didn't really work out, great for watching, tough for playing.
 * So get rid of it.
 * Also, lots of javadoc.
 *
 * Revision 1.22  2009/01/22 05:01:26  dick
 * add getXO().
 *
 * Revision 1.21  2009/01/20 07:41:54  dick
 * Send a string to the parser to set the initial positions.
 *
 * Revision 1.20  2009/01/18 05:26:24  dick
 * Javadoc.
 *
 * Revision 1.19  2009/01/18 04:43:41  dick
 * Javadoc.
 *
 * Revision 1.18  2009/01/10 05:20:36  dick
 * Turn off DEBUG.
 *
 * Revision 1.17  2009/01/10 05:02:27  dick
 * Totally cleanup who is who.  I parse the board so that You is always Player0 and playing 24 to 1.
 * Get rid of unneeded setters.  Only parseFibsBoard can set attributes.
 *
 * Revision 1.16  2009/01/09 06:56:07  dick
 * Change whosTurn parsing.
 * Turn off DEBUG.
 *
 * Revision 1.15  2009/01/08 18:25:26  dick
 * Add getMatchLength
 *
 * Revision 1.14  2009/01/08 01:28:13  dick
 * getPlayerName returns the list of players in this match.
 *
 * Revision 1.13  2009/01/06 08:05:07  dick
 * Javadoc.
 *
 * Revision 1.12  2008/12/20 23:01:39  dick
 * Support for Own Info.
 *
 * Revision 1.11  2008/12/17 04:37:11  dick
 * Add the direction that the user is playing in.
 *
 * Revision 1.10  2008/12/13 06:15:33  dick
 * Parse home and the bar.
 *
 * Revision 1.9  2008/12/12 14:44:22  dick
 * Fibs board parsing.
 *
 * Revision 1.8  2008/12/12 01:19:52  dick
 * Start reworking for a point based board,  not a checker based board.
 *
 * Revision 1.7  2008/12/11 20:24:23  dick
 * Early parsing of FIBS board.
 *
 * Revision 1.6  2008/12/10 21:55:14  dick
 * Add dice attributes.
 *
 * Revision 1.5  2008/12/09 01:47:05  dick
 * Define WHITE and BLACK.
 *
 * Revision 1.4  2008/12/08 10:16:31  dick
 * Add setStartPositions().
 *
 * Revision 1.3  2008/12/08 06:44:07  dick
 * checkers is an array of int.
 *
 * Revision 1.2  2008/03/30 05:38:51  dick
 * More skeleton checkins.
 *
 * Revision 1.1  2008/03/29 08:57:43  dick
 * BuckoFIBS skeleton checkin.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.board;

import java.util.LinkedList;

import com.buckosoft.fibs.domain.SavedMatch;


/** Define a backgammon board. <br>
 * In an MVC model, Board would be the document.
 * @author Dick Balaska
 * @since 2008/03/29
 * @version $Revision: 1.31 $ <br> $Date: 2009/02/14 12:04:30 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/board/Board.java">cvs Board.java</a>
 */
public class Board {
	private final static boolean DEBUG = true;

	/** The player playing the 'O' pieces.
	 * The value is 0. */
	public final static int O = 0;
	/** The player playing the 'X' pieces.
	 * The value is 1. */
	public final static int X = 1;

	private	int whosTurn = 0;
	private	int	cube = 1;

	/** Who has the doubling cube?<br>
	 * -1 = noone, X = X, O = O */
	private	int	hasCube = 0;
	private	int[] mayDouble = new int[2];
	private boolean[] wasDoubled = new boolean[2];
	private int matchLength = 0;
	private	int	xo;
	private	int matchScore[] = new int[2];
	private	String	initialPositions = "board:Player0:Player1:3:0:0:0:-2:0:0:0:0:5:0:3:0:0:0:-5:5:0:0:0:-3:0:-5:0:0:0:0:2:0:1:0:0:0:0:1:1:1:0:1:-1:0:25:0:0:0:0:2:0:0:0";
//	private	boolean	gameOver = true;
	private	int[][] dice = new int[2][2];
	private	String[] playerName = new String[2];
	private int[] points = new int[26];		// 0 and 25 are unused.  See bar
	private int[] bar = new int[2];
	private int[] home = new int [2];
	private int[] pipCount = new int[2];

	private	int direction;
	private int canMove;
	
	// These are really a "user settings" object and not part of the board.
	// OTOH, they are game settings.
	private	String	name = "?";
	private boolean allowPip;
	private	boolean autoBoard;
	private	boolean autoDouble;
	private boolean autoMove;
	private	boolean	away;
	private	boolean bell;
	private boolean crawford;
	private	boolean	askDouble;
	private	int		experience;
	private	boolean greedy;
	private	boolean moreBoards;
	private	boolean moves;
	private	boolean notify;
	private	double	rating;
	private	boolean ratings;
	private	boolean ready;
	private	int		redoubles;
	private	boolean	report;
	private	boolean	silent;
	private	String	timezone;
	
	private	boolean	yourTurnToRollOrDouble = false;

	private	LinkedList<SavedMatch>	savedMatches = new LinkedList<SavedMatch>();
	// Reuse SavedMatch Object
	private	LinkedList<SavedMatch>	missManners = new LinkedList<SavedMatch>();

	
	/** Called at the beginning of a game, resets game user attributes
	 */
	public void resetGame() {
		this.greedy = false;
		this.askDouble = true;
		this.wasDoubled[0] = false;
		this.wasDoubled[1] = false;
		this.yourTurnToRollOrDouble = false;
	}

	/** Fetch the points array.
	 * @return The points array.
	 */
	public int[] getPoints() {
		return(points);
	}

	/** Get the number of checkers on the bar for both players.
	 * @return The bar
	 */
	public int[] getBar() {
		return(bar);
	}

	/** Get the number of checkers in both player's homes.
	 * @return The home array
	 */
	public int[] getHome() {
		return(home);
	}

	/** Is there a pending double on the table?
	 * @return an array of wasDoubled
	 */
	public boolean[] getWasDoubled() {
		return(this.wasDoubled);
	}

	/** Set and clear the state where You push a double to Your opponent.
	 * @param doubleFlag true = you have doubled
	 */ 
	public void setYouDouble(boolean doubleFlag) {
		this.yourTurnToRollOrDouble = false;
		this.wasDoubled[0] = doubleFlag;
	}

	/** We have been doubled and need to accept or decline
	 */
	public void setAcceptDeclineDouble() {
		this.wasDoubled[1] = true;
	}

	/** Are you in a state where you are required to accept or decline a double?
	 * @return true = opponent has doubled
	 */
	public boolean isAcceptDeclineDouble() {
		return(this.wasDoubled[1]);
	}

	/** Is it your turn?
	 * @return the yourTurnToRollOrDouble
	 */
	public boolean isYourTurnToRollOrDouble() {
		return yourTurnToRollOrDouble;
	}

	/** Set your turn to roll or double
	 * @param yourTurnToRollOrDouble the yourTurnToRollOrDouble to set
	 */
	public void setYourTurnToRollOrDouble(boolean yourTurnToRollOrDouble) {
		this.yourTurnToRollOrDouble = yourTurnToRollOrDouble;
	}

	/** Are You allowed to double?
	 * @return true is yes.
	 */
	public boolean isYouMayDouble() {
		return (this.mayDouble[0] == 1);
	}

	/** Are you playing or just watching?
	 * @return true if playing
	 */ 
	public boolean isYouPlaying() {
		return(this.playerName[0].equals("You"));
	}

	/** Is it your turn to move?  That is, have you rolled and should move checkers?
	 * @return whether there is valid dice on the table.
	 */
	public boolean isYourMove() {
		return(dice[0][0] != 0);
	}

	/** Get the list of player names who are playing this game.
	 * @return The player Names
	 */
	public String[] getPlayerName() {
		return(playerName);
	}

	/** Get the number of points in the match
	 * @return The matchLength
	 */
	public int	getMatchLength() {
		return(matchLength);
	}

	/** Get the score for both players in this match
	 * @return the score array
	 */
	public int[] getMatchScore() {
		return(matchScore);
	}

	/** Return the userid that this player logged in as.
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the allowPip
	 */
	public boolean isAllowPip() {
		return allowPip;
	}

	/**
	 * @return the autoBoard
	 */
	public boolean isAutoBoard() {
		return autoBoard;
	}

	/**
	 * @param autoBoard the autoBoard to set
	 */
	public void setAutoBoard(boolean autoBoard) {
		this.autoBoard = autoBoard;
	}

	/**
	 * @return the autoDouble
	 */
	public boolean isAutoDouble() {
		return autoDouble;
	}

	/**
	 * @return the autoMove
	 */
	public boolean isAutoMove() {
		return autoMove;
	}

	/** Have You tagged yourself as away?
	 * @return true if You are not accepting invitations
	 */
	public boolean isAway() {
		return away;
	}

	/** Like, totally ignored.
	 * @return the bell
	 */
	public boolean isBell() {
		return bell;
	}

	/** Is this a crawford game?
	 * @return the crawford
	 */
	public boolean isCrawford() {
		return crawford;
	}

	/** Does the server push dice at you or wait to see if you want to roll or double
	 * @return the askDouble
	 */
	public boolean isAskDouble() {
		return this.askDouble;
	}

	public void setAskDouble(boolean askDoubles) {
		this.askDouble = askDoubles;
	}

	/** How many games have You played?
	 * @return the experience
	 */
	public int getExperience() {
		return experience;
	}

	/** Are You using greedy bearoffs?
	 * @return the greedy
	 */
	public boolean isGreedy() {
		return greedy;
	}

	/** This is pretty darn useful for client programs, so if you see it toggled off you probably want to toggle it back on.
	 * @return the moreBoards
	 */
	public boolean isMoreBoards() {
		return moreBoards;
	}

	/**
	 * @return the moves
	 */
	public boolean isMoves() {
		return moves;
	}

	/** Are we being notified when new users log on and off?
	 * @return the notify
	 */
	public boolean isNotify() {
		return notify;
	}

	/** What is Your FIBS rating?
	 * @return the rating
	 */
	public double getRating() {
		return rating;
	}

	/**
	 * @return the ratings
	 */
	public boolean isRatings() {
		return ratings;
	}

	/** Are you ready to play?
	 * @return the ready
	 */
	public boolean isReady() {
		return ready;
	}

	/** Set whether we are ready to play
	 * @param ready the ready to set
	 */
	public void setReady(boolean ready) {
		this.ready = ready;
	}

	/** Je ne comprends pas
	 * @return the redoubles
	 */
	public int getRedoubles() {
		return redoubles;
	}

	/** Are we being notified about the starting and ending of other matches? 
	 * @return the report
	 */
	public boolean isReport() {
		return report;
	}

	/** Do we receive the shout messages (general chat)?<br>
	 * Note that BuckoFIBS always receives the shouts and filters them from display.
	 * @return the silent
	 */
	public boolean isSilent() {
		return silent;
	}

	/** Get your timezone.
	 * @return the timezone
	 */
	public String getTimezone() {
		return timezone;
	}

	/** Place all checkers on their start positions
	 */
	public void setStartPositions() {
		this.parseFibsBoard(initialPositions);
	}

	/** Get whether you are playing as X or O
	 * @return -1 if you are X,  1 if you are O 
	 */
	public int	getXO() {
		return(xo);
	}

	/** When you pick up the dice, 0 them out so there is a visual cue.
	 * Sometimes it takes fibs a while to send a new board. 
	 */
	public void clearDice() {
		this.dice[0][0] = 0;
		this.dice[0][1] = 0;
	}

	/** Get who's turn it is.  -1 = X, 1 = O, 0 = game over
	 * @return the whosTurn
	 */
	public int getWhosTurn() {
		return whosTurn;
	}

	/** Get the number displayed on the double cube
	 * @return the cube
	 */
	public int getCube() {
		return cube;
	}

	/** Who has the doubling cube?
	 * -1 = noone, X = X, O = O 
	 * @return who has the cube
	 */
	public int getHasCube() {
		return hasCube;
	}

	/** -1 If you play from position 24 to position 1.<br>
	 *   1 If you play from position 1 to position 24.
	 * @return the direction
	 */
	public int getDirection() {
		return direction;
	}

	/** Get the 2x2 array of both player's both dice
	 * @return the dice
	 */
	public int[][] getDice() {
		return dice;
	}

	/** handle the '* wins the match' message. <br>
	 * Fibs doesn't send a final board, which should have the final score.
	 * So we have to derive the match over scenario.
	 */
	public void onMatchOver(String winner, String score) {
		String[] ss = score.split("\\-");
		int s0 = Integer.parseInt(ss[0]);
		int s1 = Integer.parseInt(ss[1]);
		if (playerName[0].equals(winner)) {
			matchScore[0] = s1;
			matchScore[1] = s0;
		} else {
			matchScore[0] = s0;
			matchScore[1] = s1;
		}
		whosTurn = 0;
	}

	/** Calculate and return the number of pips for both players.
	 * Used by the {@link com.buckosoft.fibs.BuckoFIBS.gui.board.MatchStatusPanel}.
	 * @return A reference to our local pipCount (reuse object).
	 */
	public int[] getPipCount() {
		int i;
		this.pipCount[O] = 0;
		this.pipCount[X] = 0;
		for (i=1; i<=24; i++) {
			if (this.points[i] < 0)
				pipCount[O] += ((this.getDirection() == -1 ? i : (25 - i)) * -this.points[i]);
			else if (this.points[i] > 0)
				pipCount[X] += ((this.getDirection() == 1 ? i : (25 - i)) * this.points[i]);
		}
		pipCount[O] += (bar[O]*25);
		pipCount[X] += (bar[X]*25);
		
		return(this.pipCount);
	}

	/** If you have a saved match against this player, this is a tooltip string describing it.
	 * @param playerName The player to check for saved matches.
	 * @return A tooltip String or null if no saved match found
	 */ 
	public String	getSavedMatchString(String playerName) {
		for (SavedMatch sm : savedMatches) {
			if (sm.getName().equals(playerName)) {
				StringBuffer sb = new StringBuffer();
				sb.append("Saved Match. Length: ");
				sb.append(sm.getMatchLength());
				sb.append(" Score: (you) ");
				sb.append(sm.getScore0());
				sb.append(" - (");
				sb.append(sm.getName());
				sb.append(") ");
				sb.append(sm.getScore1());
				return(sb.toString());
			}
		}
		return(null);
	}

	/** Parse a saved match line from FIBS.
	 * Store the match in our local list and return it
	 * @param s The string
	 * @return The parsed SavedMatch
	 */
	public SavedMatch parseSavedMatch(String s) {
		try {
			String[] ss = s.split(" +");
			SavedMatch sm = new SavedMatch();
			if (ss[0].startsWith("**")) {
				sm.setName(ss[0].substring(2));
				sm.setMatchLength(Integer.parseInt(ss[1]));
				sm.setScore0(Integer.parseInt(ss[2]));
				sm.setScore1(Integer.parseInt(ss[4]));				
			} else if (ss[1].startsWith("*")){
				sm.setName(ss[1].substring(1));
				sm.setMatchLength(Integer.parseInt(ss[2]));
				sm.setScore0(Integer.parseInt(ss[3]));
				sm.setScore1(Integer.parseInt(ss[5]));
			} else {
				sm.setName(ss[1]);
				sm.setMatchLength(Integer.parseInt(ss[2]));
				sm.setScore0(Integer.parseInt(ss[3]));
				sm.setScore1(Integer.parseInt(ss[5]));
				
			}
			this.savedMatches.add(sm);
			return(sm);
		} catch (Exception ex) {
			throw new RuntimeException("Failed to parseSavedMatch '" + s + "'", ex);
		}
	}

	/** If MissManners issued a warning about a player, this is a tooltip string describing it.
	 * @param playerName The player to check for a MissManners warning.
	 * @return A tooltip String or null if no warning found
	 */
	public String	getMissMannersString(String playerName) {
		for (SavedMatch sm : missManners) {
			if (sm.getName().equals(playerName)) {
				StringBuffer sb = new StringBuffer();
				sb.append("MissManners warns: ");
				sb.append(playerName);
				sb.append(" has ");
				sb.append(sm.getMatchLength());
				sb.append(" saved games");
				return(sb.toString());
			}
		}
		return(null);
	}

	/** Parse a MissManners warning line from FIBS.
	 * Store the warning in our local list and return it
	 * @param s The string
	 * @return The parsed SavedMatch
	 */
	public SavedMatch parseMissManners(String s) {
		try {
			String[] ss = s.split(" +");
			if (!ss[1].equals("MissManners"))
				return(null);
			SavedMatch sm = new SavedMatch();
			sm.setName(ss[4]);
			sm.setMatchLength(Integer.parseInt(ss[6]));
			this.missManners.add(sm);
			return(sm);
		} catch (Exception ex) {
			throw new RuntimeException("Failed to parseMissManners '" + s + "'", ex);
		}
	}
	
	// positions of the fields in a board line
	private final static int YOU = 1;
	private final static int SOMEPLAYER = 2;
	private final static int MATCHLENGTH = 3;
	private final static int CURRENTSCORES = 4;
	private final static int BOARD = 6;
	private final static int TURN = 32;
	private final static int DICE = 33;
	private final static int CUBE = 37;
	private final static int MAYDOUBLE = 38;
	private final static int WASDOUBLED = 40;
	private final static int XO = 41;
	private final static int DIRECTION = 42;
	private final static int ONHOME = 45;
	private final static int ONBAR = 47;
	private final static int CANMOVE = 49;

	/** Parse the board state from Fibs.
	 * Opening board looks like this:<br>
	 * board:You:someplayer:3:0:0:0:-2:0:0:0:0:5:0:3:0:0:0:-5:5:0:0:0:-3:0:-5:0:0:0:0:2:0:1:6:2:0:0:1:1:1:0:1:-1:0:25:0:0:0:0:2:0:0:0
	 * @param s The board
	 * @see <a href="http://www.fibs.com/fibs_interface.html#board_state" target="_top">FIBS Client Protocol Detailed Specification</a>
	 * <pre>
	 * 0                                   1                    2                     3                   4                     5
	 * 0     1   2          3 4 5 6 7  8 9 0 1 2 3 4 5 6 7 8  9 0 1 2 3  4 5  6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2  3 4  5 6 7 8 9 0 1 2 
	 * board:You:someplayer:3:0:0:0:-2:0:0:0:0:5:0:3:0:0:0:-5:5:0:0:0:-3:0:-5:0:0:0:0:2:0:1:6:2:0:0:1:1:1:0:1:-1:0:25:0:0:0:0:2:0:0:0
	 * </pre>
	 */
	public void parseFibsBoard(String s) {
		int i;
		String[] t = s.split(":");
		xo = Integer.parseInt(t[XO]);
		direction = Integer.parseInt(t[DIRECTION]);
		playerName[0] = t[YOU];
		playerName[1] = t[SOMEPLAYER];
		matchLength = Integer.parseInt(t[MATCHLENGTH]);
		matchScore[0] = Integer.parseInt(t[CURRENTSCORES+0]);
		matchScore[1] = Integer.parseInt(t[CURRENTSCORES+1]);
		for (i=1; i<=24; i++) {
			this.points[i] = Integer.parseInt(t[BOARD+i]) * -xo;
		}
		this.whosTurn = Integer.parseInt(t[TURN]);
		setDice(dice[O], Integer.parseInt(t[DICE+0]), Integer.parseInt(t[DICE+1]));
		setDice(dice[X], Integer.parseInt(t[DICE+2]), Integer.parseInt(t[DICE+3]));
		cube = Integer.parseInt(t[CUBE]);
		mayDouble[0] = Integer.parseInt(t[MAYDOUBLE+0]);
		mayDouble[1] = Integer.parseInt(t[MAYDOUBLE+1]);
		wasDoubled[1] = t[WASDOUBLED].equals("1") ? true : false;
		if (DEBUG)
			System.out.println("mayDouble[" + mayDouble[0] + "][" + mayDouble[1] + "]");
		if (mayDouble[0] == 1 && mayDouble[1] == 1)
			hasCube = -1;
		else if (mayDouble[0] == 1)
			hasCube = X;
		else if (mayDouble[1] == 1)
			hasCube = O;
		home[O] = Integer.parseInt(t[ONHOME+0]);
		home[X] = Integer.parseInt(t[ONHOME+1]);
		bar[O] = Integer.parseInt(t[ONBAR+0]);
		bar[X] = Integer.parseInt(t[ONBAR+1]);
		canMove = Integer.parseInt(t[CANMOVE]);
		if (DEBUG)
			System.out.println("xo=" + xo + " direction=" + direction + " canMove=" + canMove
					+ " wasDoubled[1]=" + wasDoubled[1] + " turn=" + this.whosTurn);
	}

	/** Parse the Own Info command from FIBS.
	 * @param s The Own Info line received from FIBS
	 * @see <a href="http://www.fibs.com/fibs_interface.html#clip_own_info" target="_top">FIBS Client Protocol Detailed Specification</a>
	 * <pre>
	 * 0 1    2        3         4          5        6    7    8        9      10         11     12         13    14     15     16      17    18        19     20     21
	 * 2 name allowpip autoboard autodouble automove away bell crawford double experience greedy moreboards moves notify rating ratings ready redoubles report silent timezone
	 * 2 myself 1 1 0 0 0 0 1 1 2396 0 1 0 1 3457.85 0 0 0 0 0 Australia/Melbourne
	 * </pre>
	 */
	public void parseOwnInfo(String s) {
		String[] t = s.split(" ");
		this.name		= t[1];
		this.allowPip	= Integer.parseInt(t[2]) == 1 ? true : false;
		this.autoBoard	= Integer.parseInt(t[3]) == 1 ? true : false;
		this.autoDouble	= Integer.parseInt(t[4]) == 1 ? true : false;
		this.autoMove	= Integer.parseInt(t[5]) == 1 ? true : false;
		this.away		= Integer.parseInt(t[6]) == 1 ? true : false;
		this.bell		= Integer.parseInt(t[7]) == 1 ? true : false;
		this.crawford	= Integer.parseInt(t[8]) == 1 ? true : false;
		this.askDouble	= Integer.parseInt(t[9]) == 1 ? true : false;
		this.experience = Integer.parseInt(t[10]);
		this.greedy		= Integer.parseInt(t[11]) == 1 ? true : false;
		this.moreBoards	= Integer.parseInt(t[12]) == 1 ? true : false;
		this.moves		= Integer.parseInt(t[13]) == 1 ? true : false;
		this.notify		= Integer.parseInt(t[14]) == 1 ? true : false;
		this.rating		= Double.parseDouble(t[15]);
		this.ratings	= Integer.parseInt(t[16]) == 1 ? true : false;
		this.ready		= Integer.parseInt(t[17]) == 1 ? true : false;
		this.redoubles	= t[18].equals("unlimited") ? -1 : Integer.parseInt(t[19]);
		this.report		= Integer.parseInt(t[19]) == 1 ? true : false;
		this.silent		= Integer.parseInt(t[20]) == 1 ? true : false;
		this.timezone	= t[21];
	}

	/** Sort the dice and set them in the player's dice array
	 * @param d The dice array to store in.
	 * @param d0 value of die 0
	 * @param d1 value of die 1
	 */
	private void setDice(int[]d, int d0, int d1) {
		d[0] = d0;
		d[1] = d1;
		if (d1 > d0) {
			d[0] = d1;
			d[1] = d0;
		}
	}
}
