/******************************************************************************
 * ClientAdapter.java - Hook up the network to the application
 * $Id: ClientAdapter.java,v 1.3 2009/02/14 12:27:44 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2009 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ClientAdapter.java,v $
 * Revision 1.3  2009/02/14 12:27:44  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/01/26 17:35:15  dick
 * Push the CookieMonster down to the ClientConnection.  ClientAdapter now emits the cookie with the string.
 * There are many messages besides the known "BadBoard" messages that are a result of runon messages (missing crlf).
 * CookieMonster is going to need to do a full regexp scan and see if there is anything on the right that needs to be pushed back as another message.
 *
 * Revision 1.1  2009/01/12 07:45:47  dick
 * Hook up the network to the application.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.net;

/** Hook up the network to the application.
 * 
 * @author Dick Balaska
 * @since 2009/01/11
 * @version $Revision: 1.3 $ <br> $Date: 2009/02/14 12:27:44 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/net/ClientAdapter.java">cvs ClientAdapter.java</a>
 */
public interface ClientAdapter {
	/** Define what kind of message this is
	 */
	enum MessageRoute {
		NETWORKOUT,
		SYSTEM,
		DEBUG
	}

	/** Write a message
	 * @param route What kind of message is this
	 * @param s The message to write
	 */
	void writeSystemMessage(MessageRoute route, String s);

	/** Write a message with an eol at the end
	 * @param route What kind of message is this
	 * @param s The message to write
	 */
	void writeSystemMessageln(MessageRoute route, String s);

	/** Dispatch a received message (line) from FIBS
	 * @param cookie The parsed {@link FIBSMessages} cookie
	 * @param s The line received
	 */
	void dispatch(int cookie, String s);
}
